//
//  DataQueue.m
//  WotControl
//
//  Created by wang  chao on 12-3-22.
//  Copyright (c) 2012年 bupt. All rights reserved.
//

#import "DataQueue.h"

@implementation HandlerHolder

@synthesize handler = _handler;

+ (id)holderWithHandler:(DataHandler)handler{
    HandlerHolder* holder = [[[HandlerHolder alloc] init] autorelease];
    holder.handler = handler;
    return holder;
}

- (void)dealloc{
    [_handler release];
    [super dealloc];
}

@end

static long long _currentObjID = 1;

@implementation DataQueueObj

@synthesize handlers = _handlers;
@synthesize useCache = _useCache;
@synthesize key = _key;
@synthesize params = _params;
@synthesize keyPath = _keyPath;
@synthesize objID = _objID;

- (id)initWithKey:(NSString *)key params:(NSDictionary *)params handler:(DataHandler)handler useCache:(BOOL)useCache{
    if ([super init]) {
        DataManager* dm = [DataManager manager];
        _useCache = useCache;
        _handlers = [[NSMutableArray arrayWithObject:[HandlerHolder holderWithHandler:handler]] retain];
        _key = [key retain];
        _params = [params retain];
        _keyPath = [[dm keyPath:key params:params] retain];
        _objID = _currentObjID;
        _currentObjID ++;
        if (!_keyPath) {
            @throw [NSException exceptionWithName:@"NoMapperException" reason:@"No Mapper" userInfo:nil];
        }
    }
    return self;
}

- (void)dealloc{
    [_handlers release];
    [_key release];
    [_params release];
    [_keyPath release];
    [super dealloc];
}

@end

@interface DataQueue (Private)

- (DataQueueAlgorithmObj*)_queObjForKey:(NSString*)key params:(NSDictionary*)params useCache:(BOOL)useCache;
- (void)_runTasks:(NSArray*)tasks;

@end

@implementation DataQueue

@synthesize isMerge = _isMerge;
@synthesize queue  = _queue;
@synthesize algorithm = _algorithm;

- (id)init{
    if ([super init]) {
        _isMerge = YES;
        self.queue = [NSMutableArray arrayWithCapacity:20];
        self.algorithm = [[[DataQueueAlgorithm alloc] init] autorelease]; 
    }    
    return self;
}

- (id)initWithMaxTask:(NSInteger)max{
    if([self init]){
        self.algorithm.maxTask = max;
    }
    return self;
}

- (void)getValue:(NSString *)key params:(NSDictionary *)params handler:(DataHandler)handler{
    [self getValue:key params:params handler:handler useCache:YES];
}

- (void)getValue:(NSString *)key params:(NSDictionary *)params handler:(DataHandler)handler useCache:(BOOL)useCache{
    DataQueueAlgorithmObj* aobj = nil;
    DataQueueObj* obj = nil;
    DataManager* dm = [DataManager manager];
    
    if (useCache) {
        DataInfo* info = [dm getCache:key params:params];
        if (info) {
            if (handler) {
                handler(info,nil);
            }
            return;
        }
    }

    if (_isMerge) {
        aobj = [self _queObjForKey:key params:params useCache:useCache];
        obj = aobj.obj;
        if (obj && handler) {
            [obj.handlers addObject:[HandlerHolder holderWithHandler:handler]];
        }
    }
    
    if (!aobj) {
        DataQueueObj* obj = [[[DataQueueObj alloc] initWithKey:key params:params handler:handler useCache:useCache] autorelease];
        DataQueueAlgorithmObj* aobj = [[[DataQueueAlgorithmObj alloc] init] autorelease];
        aobj.obj = obj;
        
        NSArray* tasks = [_algorithm addTask:aobj];
        [self _runTasks:tasks];
    }
}

- (void)getCacheAndValue:(NSString *)key params:(NSDictionary *)params handler:(DataHandler)handler{
    DataManager* dm = [DataManager manager];
    DataInfo* info = [dm getCache:key params:params];
    if (info) {
        if (handler) {
            handler(info,nil);
        }
    }
    [self getValue:key params:params handler:handler useCache:NO];
}

- (void)_runTasks:(NSArray *)tasks{
    DataManager* dm = [DataManager manager];
    for (DataQueueAlgorithmObj* task in tasks) {
        DataQueueObj* taskobj = task.obj;
        [_queue addObject:task];
        [dm getValue:taskobj.key params:taskobj.params handler:^(DataInfo* info,NSError* error){
            for (HandlerHolder* holder in taskobj.handlers) {
                if (holder.handler) {
                    holder.handler(info,error);
                    holder.handler = NULL;
                }
            }
            [_queue removeObject:task];
            NSArray* nextTasks = [_algorithm finishTask:task];
            dispatch_async(dispatch_get_main_queue(), ^{
                [self _runTasks:nextTasks];
            });
        }useCache:taskobj.useCache];
    }
}

- (DataQueueAlgorithmObj*)_queObjForKey:(NSString *)key params:(NSDictionary *)params useCache:(BOOL)useCache{
    DataManager* dm = [DataManager manager];
    NSString* keyPath = [dm keyPath:key params:params];
    if (!keyPath) {
        return nil;
    }
    for (DataQueueAlgorithmObj* aobj in _queue) {
        DataQueueObj* obj = aobj.obj;
        if ([obj.keyPath isEqualToString:keyPath]&&obj.useCache == useCache) {
            return aobj;
        }
    }
    return nil;
}

- (void)dealloc{
    [_queue release];
    [_algorithm release];
    [super dealloc];
}

@end
