//
//  LocalStorageDriver.m
//  DataManager
//
//  Created by wang  chao on 12-2-13.
//  Copyright 2012年 bupt. All rights reserved.
//

#import "LocalDriver.h"
static NSString* _head = @"_datamanager_";

@interface LocalDriver (Private)

- (NSString*)_keyPath:(DataMapper*)mapper params:(NSDictionary*)params;
- (NSString*)_keyHead:(NSString*)key;
    
@end

@implementation LocalDriver (Private)

- (NSString*)_keyHead:(NSString *)key{
    return [_head stringByAppendingFormat:@"_%@",key];
}

- (NSString*)_keyPath:(DataMapper*)mapper params:(NSDictionary*)params{
    NSString* key = mapper.key;
    return [NSString stringWithFormat:@"%@_%@",[self _keyHead:key],[mapper keyPathForParams:[DataMapper normalParams:params]]];
}

@end

@implementation LocalDriver

@synthesize storeType = _storeType;

- (id)init
{
    self = [super init];
    if (self) {
        _storeType = DISK;
    }
    return self;
}

- (id)initWithType:(StoreType)type{
    self = [self init];
    if (self) {
        _storeType = type;
    }
    return self;
}

- (BOOL)store:(DataInfo*)data mapper:(DataMapper *)mapper params:(NSDictionary *)params{
    NSString* keyPath = [self _keyPath:mapper params:params];
    LocalStorage* localStorage = [LocalStorage instance];
    return [localStorage store:[data dictionaryValue] forKey:keyPath storeType:_storeType];
}

- (BOOL)deleteData:(DataMapper *)mapper params:(NSDictionary *)params{
    NSString* keyPath = [self _keyPath:mapper params:params];
    LocalStorage* localStorage = [LocalStorage instance];
    return [localStorage delete:keyPath storeType:_storeType];
}

- (DataInfo*)getData:(DataMapper *)mapper params:(NSDictionary *)params{
    NSString* keyPath = [self _keyPath:mapper params:params];
    LocalStorage* localStorage = [LocalStorage instance];
    DataInfo* info = nil;
    NSDictionary* dataDict = [localStorage getValue:keyPath storeType:_storeType];
    if (dataDict) {
       info = [DataInfo dataInfoWithDictionary:dataDict];
    }
    info.key = mapper.key;
    info.params = params;
    return info;
}

- (void)clean{
    LocalStorage* localStorage = [LocalStorage instance];
    [localStorage cleanPrefix:_head type:_storeType];
}
- (void)clean:(DataMapper*)mapper{
    LocalStorage* localStorage = [LocalStorage instance];
    [localStorage cleanPrefix:[self _keyHead:mapper.key] type:_storeType];
}

@end
